// SUPPLEMENTARY MATERIAL FOR ABREVAYA, HAUSMAN, AND KHAN ECONOMETRICA PAPER 
// "TESTING FOR CAUSAL EFFECTS IN A GENERALIZED REGRESSION MODEL WITH ENDOGENOUS REGRESSORS"
// OX CODE FOR MONTE CARLO SIMULATIONS (DESIGN 3, TABLE S3 IN PAPER)

#include <oxstd.h>
#import <maximize>

decl Z1, Z2, Y1, Y2;

// sign function: 1 if positive, 0 otherwise
sign(v)
{
	return( (v.>0)-(v.<0) );
}

// Kendall's tau (rank correlation statistic) --- used for test with no additional covariates
tau_function()
{
	decl i, j, numer, denom, nobs, tau_stat, zeta_val, temp;

	numer = 0;
	denom = 0;
	nobs = rows(Y1);
	
	for (i=0; i<nobs-1; i++)
		for (j=i+1; j<nobs; j++)
		{
			denom ++;
			numer += sign(Y1[i]-Y1[j])*sign(Z2[i]-Z2[j]);
		}

	tau_stat = numer/denom;

	zeta_val = 0;
	for (i=0; i<nobs-1; i++)
	{
		temp = sign(Y1[i]-Y1)'*sign(Z2[i]-Z2);
		zeta_val += ((temp/(nobs-1))-tau_stat)^2;
	}
	zeta_val = zeta_val/nobs;

	return(tau_stat|(2*sqrt(zeta_val/nobs)));
}

// function to make data --- see paper for design details
makedata(const nobs, const rho, const beta_value)
{
	decl u1, v2;
	
	Z1 = ones(nobs,1);

	Z2 = ((rann(nobs,1).^2)+(rann(nobs,1).^2)+(rann(nobs,1).^2))/sqrt(6);   // normalized chi-square 3 d.f.

	v2 = rann(nobs,1);
	u1 = rho*v2 + sqrt(1-rho^2)*rann(nobs,1);

	Y2 = -1 + sqrt(Z2/5) + v2;
	
	Y1 = (5*beta_value*Y2 + u1 .> 0);
	
	return(1);
}

main()
{
	decl ir, delta2, dfunc, mhess, cov_delta2, nobs;
	decl delta1, cov_delta1;
	decl beta_iv, cov_iv, eps_iv, sigma2_iv;
	decl temp_tau;
	decl outvec = 0;
	decl num_simul = 0;
	decl simul_type = 0;
	decl rho;
	decl beta_value;
	decl total_simuls = 1000;
	decl total_simul_types = 16;
	decl i, temp_matrix, xhat;
	
	for (simul_type = 1; simul_type<=total_simul_types; simul_type++)
	{
		println("Simulation design ", simul_type);

		outvec=0;
		if (simul_type == 1)
		{
			rho = 0;
			beta_value = 0;
		}
		else if (simul_type == 2)
		{
			rho = 0.25;
			beta_value = 0;
		}
		else if (simul_type == 3)
		{
			rho = 0.50;
			beta_value = 0;
		}
		else if (simul_type == 4)
		{
			rho = 0.75;
			beta_value = 0;
		}
		else if (simul_type == 5)
		{
			rho = 0;
			beta_value = 0.1;
		}
		else if (simul_type == 6)
		{
			rho = 0.25;
			beta_value = 0.1;
		}
		else if (simul_type == 7)
		{
			rho = 0.50;
			beta_value = 0.1;
		}
		else if (simul_type == 8)
		{
			rho = 0.75;
			beta_value = 0.1;
		}
		else if (simul_type == 9)
		{
			rho = 0;
			beta_value = 0.2;
		}
		else if (simul_type == 10)
		{
			rho = 0.25;
			beta_value = 0.2;
		}
		else if (simul_type == 11)
		{
			rho = 0.50;
			beta_value = 0.2;
		}
		else if (simul_type == 12)
		{
			rho = 0.75;
			beta_value = 0.2;
		}
		else if (simul_type == 13)
		{
			rho = 0;
			beta_value = 0.3;
		}
		else if (simul_type == 14)
		{
			rho = 0.25;
			beta_value = 0.3;
		}
		else if (simul_type == 15)
		{
			rho = 0.50;
			beta_value = 0.3;
		}
		else if (simul_type == 16)
		{
			rho = 0.75;
			beta_value = 0.3;
		}

		nobs = 200;

		for (num_simul = 1; num_simul<=total_simuls; num_simul++)
		{
			makedata(nobs,rho,beta_value);

			// IV estimation
			beta_iv = invert((Z1~Z2)'(Z1~Y2))*((Z1~Z2)'Y1);

			eps_iv = Y1-(Z1~Y2)*beta_iv;
			xhat = (Z1~Z2)*(invert((Z1~Z2)'(Z1~Z2))*(Z1~Z2)'(Z1~Y2));
			temp_matrix = zeros(columns(xhat),columns(xhat));
			for (i=0; i<nobs; i++)
				temp_matrix += eps_iv[i]*eps_iv[i]*xhat[i][]'xhat[i][];
			cov_iv = invert(xhat'xhat)*temp_matrix*invert(xhat'xhat);

			// OLS estimation
			decl beta_ols, eps_ols, cov_ols;
			beta_ols = invert((Z1~Y2)'(Z1~Y2))*((Z1~Y2)'Y1);

			eps_ols = Y1-(Z1~Y2)*beta_ols;
			xhat = (Z1~Y2);
			temp_matrix = zeros(columns(xhat),columns(xhat));
			for (i=0; i<nobs; i++)
				temp_matrix += eps_ols[i]*eps_ols[i]*xhat[i][]'xhat[i][];
			cov_ols = invert(xhat'xhat)*temp_matrix*invert(xhat'xhat);
	
			// semiparametric (kendall's tau) estimation
			temp_tau = tau_function();

			if (outvec==0)
				outvec = beta_ols[1]|sqrt(cov_ols[1][1])|beta_iv[1]|sqrt(cov_iv[1][1])|temp_tau;
			else
				outvec = outvec~(beta_ols[1]|sqrt(cov_ols[1][1])|beta_iv[1]|sqrt(cov_iv[1][1])|temp_tau);
		}
		println("rejections at 5% level", sumr( fabs(outvec[0][]./outvec[1][]).>1.96 ), " ", sumr( fabs(outvec[2][]./outvec[3][]).>1.96 ), " ", sumr( fabs(outvec[4][]./outvec[5][]).>1.96 ));
		println("rejections at 10% level", sumr( fabs(outvec[0][]./outvec[1][]).>1.645 ), " ", sumr( fabs(outvec[2][]./outvec[3][]).>1.645 ), " ", sumr( fabs(outvec[4][]./outvec[5][]).>1.645 ));
	}
}
